@@ -34,6 +34,7 @@
 #include "sysprof-local-profiler.h"
 #include "sysprof-platform.h"
 
+#include "sysprof-capture-autocleanups.h"
 #include "sysprof-control-source.h"
 #include "sysprof-gjs-source.h"
 #include "sysprof-hostinfo-source.h"
@@ -197,7 +198,8 @@ sysprof_local_profiler_finish_stopping (SysprofLocalProfiler *self)
   g_assert (priv->is_stopping == TRUE);
   g_assert (priv->stopping->len == 0);
 
-  reader = sysprof_capture_writer_create_reader (priv->writer, 0);
+  reader = sysprof_capture_writer_create_reader (priv->writer);
+  g_assert (reader != NULL);
 
   for (guint i = 0; i < priv->sources->len; i++)
     {
@@ -982,9 +984,9 @@ profiler_iface_init (SysprofProfilerInterface *iface)
   iface->stopped = sysprof_local_profiler_real_stopped;
 }
 
-static gboolean
+static bool
 find_profiler_meta_cb (const SysprofCaptureFrame *frame,
-                       gpointer                   user_data)
+                       void                      *user_data)
 {
   const SysprofCaptureMetadata *meta = (const SysprofCaptureMetadata *)frame;
   GKeyFile **keyfile = user_data;
@@ -1005,7 +1007,7 @@ find_profiler_meta_cb (const SysprofCaptureFrame *frame,
       return *keyfile == NULL;
     }
 
-  return TRUE;
+  return true;
 }
 
 SysprofProfiler *
